; SketchyLISP Extension Library
; Copyright (C) 2006 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; exists

; ---conformance---
; R5.91RS

; ---purpose---
; Test whether given property exists in a sequence of lists.
; The property is expressed using an
; .V n-ary
; function
; .V p.
; .V P
; is applied to a list consisting of the first
; member of each given list. Hence there must be
; exactly
; .V n
; lists. If
; .V p
; returns truth,
; .C exists
; returns
; .C #t
; immediately.
; Otherwise it is applied to a list consisting
; of the second member of each given list, etc.
; If
; .V p
; returns falsity for all sets of members,
; .C forall
; returns
; .C #f.
; .B
; All lists passed to
; .C forall
; must have the same length.
; .B
; When all lists passed to
; .C forall
; are empty, it returns
; .C #f.

; ---args---
; F - predicate expressing property
; A* ... - lists

; ---keywords---
; EXISTS function, existential quantifier, property, predicate

; ---see-also---
; forall, memp, filter

; ---example---
; (exists < '(5 5 5) '(5 7 5)) => #t

; (require "nullp.scm") ; null?
; (require "lib/map-car.scm")

; ---code---
(define (exists p . a*)
  (letrec
    ((carof (lambda (a)
       (map-car car a)))
     (cdrof (lambda (a)
       (map-car cdr a)))
     (_exists (lambda (a*)
       (cond ((null? (car a*)) #f)
         (else (or (apply p (carof a*))
                   (_exists (cdrof a*))))))))
    (_exists a*)))
