; SketchyLISP Extension Library
; Copyright (C) 2006 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; memp

; ---conformance---
; R5.91RS

; ---purpose---
; Extract the tail of a list whose first
; member has a given property.
; The property is expressed using a predicate.
; Evaluate to
; .C #f
; if no member satisfying the predicate exists.

; ---args---
; P - predicate
; A - list

; ---keywords---
; MEMP function, property, predicate

; ---see-also---
; member, memq, memv, remp, assp

; ---example---
; (memp pair? '(a b c (x . y) d e f)) => ((x . y) d e f)

(define memp #t)

; (require "nullp.scm") ; null?

; ---model---
; (define (memp p a)
;   (cond ((null? a) #f)
;     ((p (car a)) a)
;     (else (memp p (cdr a)))))

; ---code---
(define (memp p a)
  (letrec
    ((_memp (lambda (a)
       (cond ((null? a) #f)
         ((p (car a)) a)
         (else (_memp (cdr a)))))))
    (_memp a)))
